"""
Code to train a small regression classifier on the Yummly recipe dataset. 
Author: Carolyn Anderson
Date: 2/27/2022
"""
import pandas as pd
import math
import tensorflow as tf
from tensorflow import keras
from keras.models import Model
from keras import backend as K
from tensorflow.keras import layers
tf.keras.utils.set_random_seed(0)

def make_model(input_shape, num_classes):
    inputs = keras.Input(shape=input_shape)
    """
    INSERT MODEL CODE HERE
    """
    return keras.Model(inputs, outputs)

def make_array(dataframe):
	noNones = dataframe.applymap(lambda x:-1 if x == 'None' else x)
	toFloats = noNones.to_numpy().astype(float)
	return toFloats

def descriptive_stats(dataframe):
	dataframe = dataframe.applymap(lambda x:-1 if x == 'None' else x)
	for column_name in dataframe.columns:
		column = dataframe[column_name]
		try:
			column = column.astype(float)
			print(column_name.upper()," "*(18-len(column_name.upper())),"| Stdev:",round(column.std(),2),"\t","Mean:",round(column.mean(),2),"\t","Median:",round(column.median(),2))
		except:
			pass

def stats_by_label(dataframe):
	print(type(dataframe['category_number'][0]))
	for i in range(6):
		label_subset = dataframe.loc[dataframe['category_number'] == i]
		print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
		print("DESCRIPTIVE STATS FOR CATEGORY",i)
		descriptive_stats(label_subset)
		print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")

def print_coefficients(model,selected_features):
	print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
	print("Model weights (rows=features; columns=categories)")
	print("Feature"+" "*13+"\t0(Mains)\t1(Desserts)\t2(Beverages)\t3(Soups)\t4(Salads)\t5(Sauces)")
	weights = model.layers[1].weights[0]
	for i,f in enumerate(weights):
		ws = [float(v) for v in f]
		print(selected_features[i]+" "*(20-len(selected_features[i]))+"\t"+'\t\t'.join(map(lambda x:str(round(x,3)),ws)))

def get_predicted_label_from_predictions(predictions):
	predicted_label = max([(predictions[l],l) for l in range(len(predictions))])[1]
	return predicted_label

def sample_and_print_predictions(test_features,test_data,model):
	# Predict labels for some items from the test set:
	print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
	sample_predictions = model.predict(x=test_features[:10])
	for i,p in enumerate(sample_predictions):
		predicted_label = get_predicted_label_from_predictions(sample_predictions[i])
		print(test_data["name"][i],"was predicted to be class",predicted_label,"and is actually class",test_data["category_number"][i],"("+test_data["category"][i]+")")
		print("Predictions:",' '.join([str(s) for s in p]))
	return

def main():
	train_data = pd.read_csv('yummly_train.csv')
	test_data = pd.read_csv('yummly_test.csv')

	print(train_data.head())
	print(len(train_data))

	selected_features = ["rating", "time","servings"]

	train_features = make_array(train_data[selected_features])
	train_labels = make_array(train_data[["category_number"]])
	test_features = make_array(test_data[selected_features])
	test_labels = make_array(test_data[["category_number"]])

	epochs = 5
	batch_size = 1

	model = make_model(input_shape=len(selected_features),num_classes=6)

	callbacks = []#[keras.callbacks.ModelCheckpoint("save_at_{epoch}.h5"),] #uncomment to checkpoint
	model.compile(
    	optimizer=keras.optimizers.Adam(1e-4),
    	loss=tf.losses.SparseCategoricalCrossentropy(),
    	metrics=["accuracy"],
	)
	model.fit(
    	x=train_features,y=train_labels, epochs=epochs, \
    	batch_size=batch_size, callbacks=callbacks, validation_data=(test_features,test_labels),
	)

	# Predict labels for some items from the test set:
	sample_and_print_predictions(test_features,test_data,model)

	# Which features are most informative for each class?
	print_coefficients(model,selected_features)

	# Descriptive stats for features:
	stats_by_label(train_data)

main()